﻿/*
This file is part of Claymore.
Copyright (C) 2009, Fabrice Michellonet (fabrice dot michellonet at gmail dot com)

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/


using System;
using System.Linq;

namespace Claymore
{
    public partial class ClaymoreViewManager
    {
        
        /// <summary>
        /// Creates a view based on it's implemented interface.
        /// </summary>
        /// <typeparam name="TIView">The type of the I view.</typeparam>
        /// <returns></returns>
        public virtual TIView CreateView<TIView>()
            where TIView : IView
        {

            TIView view = default(TIView);

            Type viewType = _viewTypeName2ViewType.Values.FirstOrDefault(tp => typeof(TIView).IsAssignableFrom(tp));
            if (viewType == null)
                return view;

            view = (TIView)Activator.CreateInstance(viewType);

            StepIntoView(view);

            // if it's the first instance of the view then we store it.
            AddView(view);

            return view;
        }
    }
}
