﻿/*
This file is part of Claymore.
Copyright (C) 2010, Fabrice Michellonet (fabrice dot michellonet at gmail dot com)

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/
using System.Reflection;

namespace Claymore
{
    public interface IViewManager
    {
        /// <summary>
        /// Gets a view based on it's name.
        /// </summary>
        /// <param name="viewName">Name of the view.</param>
        /// <returns></returns>
        IView GetView(string viewName);

        /// <summary>
        /// Adds the views assembly.
        /// </summary>
        /// <param name="asm">The asm.</param>
        void AddViewsAssembly(Assembly asm);

        /// <summary>
        /// Gets a view based on it's implemented interface.
        /// </summary>
        /// <typeparam name="TIView">The type of the I view.</typeparam>
        /// <returns></returns>
        TIView GetView<TIView>()
            where TIView : IView;

        /// <summary>
        /// Creates the view based on it's name
        /// </summary>
        /// <param name="viewName">Name of the view.</param>
        /// <returns></returns>
        IView CreateView(string viewName);

        /// <summary>
        /// Creates a view based on it's implemented interface.
        /// </summary>
        /// <typeparam name="TIView">The type of the I view.</typeparam>
        /// <returns></returns>
        TIView CreateView<TIView>()
            where TIView : IView;
    }
}
